@@ -1,7 +1,7 @@
 import pytest
 from textwrap import dedent
 
-from ..compat import PY3
+from ..compat import PY3, wraps
 from ..interface import implements, InvalidImplementation, Interface, default
 
 
@@ -527,6 +527,22 @@ def test_default():
     assert C().has_default() == 3
 
 
+def test_override_default():
+
+    class IFace(Interface):  # pragma: nocover
+
+        @default
+        def default(self):
+            return 'ayy'
+
+    class C(implements(IFace)):  # pragma: nocover
+
+        def default(self):
+            return 'lmao'
+
+    assert C().default() == 'lmao'
+
+
 def test_conflicting_defaults():
 
     class IFace1(Interface):  # pragma: nocover
@@ -656,3 +672,51 @@ The following attributes may be accessed but are not part of the interface:
 
 Consider changing the implementation of default_method or making these attributes part of HasDefault."""  # noqa
     assert second == expected_second
+
+
+def test_wrapped_implementation():
+    class I(Interface):  # pragma: nocover
+        def f(self, a, b, c):
+            pass
+
+    def wrapping_decorator(f):
+        @wraps(f)
+        def inner(*args, **kwargs):  # pragma: nocover
+            pass
+
+        return inner
+
+    class C(implements(I)):  # pragma: nocover
+        @wrapping_decorator
+        def f(self, a, b, c):
+            pass
+
+
+def test_wrapped_implementation_incompatible():
+    class I(Interface):  # pragma: nocover
+        def f(self, a, b, c):
+            pass
+
+    def wrapping_decorator(f):
+        @wraps(f)
+        def inner(*args, **kwargs):  # pragma: nocover
+            pass
+
+        return inner
+
+    with pytest.raises(InvalidImplementation) as e:
+        class C(implements(I)):  # pragma: nocover
+            @wrapping_decorator
+            def f(self, a, b):  # missing ``c``
+                pass
+
+    actual_message = str(e.value)
+    expected_message = dedent(
+        """
+        class C failed to implement interface I:
+
+        The following methods of I were implemented with invalid signatures:
+          - f(self, a, b) != f(self, a, b, c)"""
+    )
+
+    assert actual_message == expected_message
